SystemOrganization addCategory: #FuelMetalevelTests!
SystemOrganization addCategory: #'FuelMetalevelTests-Mocks'!

Trait named: #FLTCreateClassOrTraitSerializationTest
	uses: {}
	category: 'FuelMetalevelTests'!

!FLTCreateClassOrTraitSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of an entire class or trait, in the cases in which they *are not* present in the image at materialization time.!

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 04:21'!
resultOfSerializeRemoveAndMaterialize: aClass

	self analyzer considerInternalAll: (Array with: aClass).
	self serialize: aClass.
	aClass removeFromSystem.
	^self materialized
! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 04:21'!
resultOfSerializeRemoveAndMaterializeAll: objects

	| classesOrTraits |
	classesOrTraits := objects select: [ :anObject | anObject isBehavior or: [anObject isTrait]].
	self analyzer considerInternalAll: classesOrTraits.
	self serialize: objects.
	classesOrTraits do: [ :aClassOrTrait | aClassOrTrait removeFromSystem ].
	^self materialized
! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 18:14'!
testCompiledMethodClassBinding
	"The class binding of each CM has to point to a correct Association, which should not be added to Smalltalk globals."
	
	| aClassOrTrait materializedClassOrTrait className method1 method2 |	
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait compileSilently: 'fortyTwo ^42' classified: #test.
	aClassOrTrait compileSilently: 'fortyThree ^43' classified: #test.
	className := aClassOrTrait name.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.
	method1 := materializedClassOrTrait compiledMethodAt: #fortyTwo.
	method2 := materializedClassOrTrait compiledMethodAt: #fortyThree.

	self assert: className = method1 methodClass name.
	self assert: className = method2 methodClass name.
	self assert: method1 classBinding == method2 classBinding.
	self deny: (Smalltalk globals includesKey: className asSymbol)! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/28/2011 20:57'!
testCreateBasic
	"Tests materialization a class or trait not defined in the image."

	| aClassOrTrait materializedClassOrTrait environment category name |
	aClassOrTrait := self newClassOrTrait.
	environment := aClassOrTrait environment.
	category := aClassOrTrait category.
	name := aClassOrTrait name.
	
	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self deny: aClassOrTrait == materializedClassOrTrait. 
	self assert: environment == materializedClassOrTrait environment.
	"Classes are not direclty inserted in the system organization so we can only test #basicCategory and not #category."
	self assert: category = materializedClassOrTrait basicCategory.
	self assert: name = materializedClassOrTrait name.

     "It is important to notice that Fuel does not add the materialized class or trait into Smalltalk globals."
    self deny: (Smalltalk globals includesKey: name).
! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 18:10'!
testCreateWithClassSideMethod
	"Tests materialization of a class-side method in a class or trait not defined in the image."

	| aClassOrTrait materializedClassOrTrait |
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait classSide compileSilently: 'fortyTwo ^42' classified: #test.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: (materializedClassOrTrait classSide includesSelector: #fortyTwo).
	self assert: #(fortyTwo) asSet = materializedClassOrTrait classSide localSelectors.
	self assert: #test = (materializedClassOrTrait classSide whichCategoryIncludesSelector: #fortyTwo).
	self assert: 42 = ((self newInstanceFrom: materializedClassOrTrait) class perform: #fortyTwo).! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 04:08'!
testCreateWithClassTrait
	"Tests materialization of a class not defined in the image, with a class trait"
	
	| aClassOrTrait materializedClassOrTrait aTrait |	
	aTrait := self newSecondaryTrait.
	aTrait classSide compileSilently: 'fortyTwo ^42'.
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait addToComposition: aTrait.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: 1 = materializedClassOrTrait traits size.
	self assert: (materializedClassOrTrait traits includes: aTrait).
	self assert: (aTrait users includes: materializedClassOrTrait).
	self assert: materializedClassOrTrait classSide localSelectors isEmpty.
	self assert: 42 = ((self newInstanceFrom: materializedClassOrTrait) class perform: #fortyTwo).! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 18:00'!
testCreateWithComment
	"Tests materialization of the comment of a class or trait not defined in the image."
	
	| aClassOrTrait materializedClassOrTrait |	
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait comment: 'test comment' stamp: 'test stamp'.
	
	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: 'test comment' = materializedClassOrTrait comment asString.
	self assert: 'test stamp' = materializedClassOrTrait organization commentStamp.! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 04:09'!
testCreateWithExternalTrait
	"Tests materialization of a class not defined in the image, with a trait"
	
	| aClassOrTrait materializedClassOrTrait aTrait |	
	aTrait := self newSecondaryTrait.
	aTrait compileSilently: 'fortyTwo ^42'.
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait addToComposition: aTrait.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: 1 = materializedClassOrTrait traits size.
	self assert: (materializedClassOrTrait traits includes: aTrait).
	self assert: (aTrait users includes: materializedClassOrTrait).
	self assert: materializedClassOrTrait localSelectors isEmpty.
	self assert: 42 = ((self newInstanceFrom: materializedClassOrTrait) perform: #fortyTwo).
! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 17:59'!
testCreateWithInstance
	"Tests materialization of an internal class or trait together with an object using it."
	
	| aClassOrTrait materializedObjects objectsToSerialize |	
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait compileSilently: 'fortyTwo ^42' classified: #test.
	objectsToSerialize := Array 
		with: aClassOrTrait 
		with: (self newInstanceFrom: aClassOrTrait).

	materializedObjects := self resultOfSerializeRemoveAndMaterializeAll: objectsToSerialize.

	self assert: (materializedObjects first includesSelector: #fortyTwo).
	self assert: 42 = (materializedObjects second perform: #fortyTwo).! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 04:09'!
testCreateWithInternalTrait
	"Tests materialization of a class not defined in the image, with a trait, both internally serialized"
	
	| aClassOrTrait aTrait serializedArray materializedArray materializedClassOrTrait materializedTrait |	
	aTrait := self newSecondaryTrait.
	aTrait compileSilently: 'fortyTwo ^42'.
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait addToComposition: aTrait.
	serializedArray := Array with: aClassOrTrait with: aTrait.

	materializedArray := self resultOfSerializeRemoveAndMaterializeAll: serializedArray.
	materializedClassOrTrait := materializedArray first.
	materializedTrait := materializedArray second.

	self assert: 1 = (materializedClassOrTrait traits size).
	self assert: (materializedClassOrTrait traits includes: materializedTrait).
	self assert: (materializedTrait users includes: materializedClassOrTrait).
	self assert: materializedClassOrTrait localSelectors isEmpty.
	self assert: 42 = ((self newInstanceFrom: materializedClassOrTrait) perform: #fortyTwo).
! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 17:58'!
testCreateWithMethod
	"Tests materialization of a compiled method in a class not defined in the image."
	
	| aClassOrTrait materializedClassOrTrait |	
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait compileSilently: 'fortyTwo ^42' classified: #test.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: (materializedClassOrTrait includesSelector: #fortyTwo).
	self assert: #(fortyTwo) asSet = materializedClassOrTrait localSelectors.
	self assert: #test = (materializedClassOrTrait whichCategoryIncludesSelector: #fortyTwo).
	self assert: 42 = ((self newInstanceFrom: materializedClassOrTrait) perform: #fortyTwo).! !

!FLTCreateClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 04:08'!
testCreateWithTraitOnClassSide
	"Tests materialization of a class not defined in the image, with a trait added to its class side"
	
	| aClassOrTrait materializedClassOrTrait aTrait |	
	aTrait := self newSecondaryTrait.
	aTrait compileSilently: 'fortyTwo ^42'.
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait classSide addToComposition: aTrait.

	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert: 1 = materializedClassOrTrait classSide traits size.
	self assert: (materializedClassOrTrait classSide traits includes: aTrait).
	self assert: (aTrait users includes: materializedClassOrTrait classSide).
	self assert: materializedClassOrTrait classSide localSelectors isEmpty.
	self assert: 42 = ((self newInstanceFrom: materializedClassOrTrait) class perform: #fortyTwo).! !

CompiledMethod variableByteSubclass: #FLCompiledMethodClassMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevelTests-Mocks'!

FLPair variableSubclass: #FLVariableClassMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevelTests-Mocks'!

Object variableByteSubclass: #FLVariableByteClassMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevelTests-Mocks'!

Object variableWordSubclass: #FLVariableWordClassMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevelTests-Mocks'!

FLSerializationTest subclass: #FLFullBasicSerializationTest
	instanceVariableNames: ''
	classVariableNames: 'ClassVariableForTesting'
	poolDictionaries: ''
	category: 'FuelMetalevelTests'!

!FLFullBasicSerializationTest methodsFor: 'running' stamp: 'MartinDias 12/31/2011 00:10'!
setUpSerializer
	super setUpSerializer.
	serializer analyzer useFullMappers.! !

!FLFullBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 02:37'!
testClassVariable

	self assertSerializationIdentityOf: (self class classPool associationAt: #ClassVariableForTesting)! !

!FLFullBasicSerializationTest methodsFor: 'tests-replicate-of-basic-TODO' stamp: 'MartinDias 12/30/2011 23:21'!
testConsiderCustomGlobal
	
	| aPerson |
	self analyzer considerGlobal: #FLGlobalVariableForTesting.

	aPerson := FLPerson new.
	Smalltalk globals at: #FLGlobalVariableForTesting put: aPerson.
	
	self assertSerializationIdentityOf: aPerson.
! !

!FLFullBasicSerializationTest methodsFor: 'tests-replicate-of-basic-TODO' stamp: 'MartinDias 12/30/2011 23:21'!
testConsiderTranscriptGlobal
	
	self analyzer considerGlobal: #Transcript.
	self assertSerializationIdentityOf: Transcript. 
! !

!FLFullBasicSerializationTest methodsFor: 'tests-replicate-of-basic-TODO' stamp: 'MartinDias 12/30/2011 23:21'!
testDontConsiderTranscriptGlobal
	
	self deny: (self resultOfSerializeAndMaterialize: Transcript) == Transcript. 
! !

!FLFullBasicSerializationTest methodsFor: 'tests-replicate-of-basic-TODO' stamp: 'MartinDias 12/30/2011 23:22'!
testGlobalClass
	"A class should be global by default."
	
	self assertSerializationIdentityOf: Integer.
! !

!FLFullBasicSerializationTest methodsFor: 'tests-replicate-of-basic-TODO' stamp: 'MartinDias 12/30/2011 23:22'!
testGlobalMetaclass
	"A metaclass should be global by default."
	
	self assertSerializationIdentityOf: Integer class.
! !

!FLFullBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/30/2011 23:43'!
testMethodDictionaries
	| methodDicts |
	methodDicts := OrderedCollection new.
	methodDicts addAll: ((SystemNavigation default allClassesInPackageNamed: 'System') 
		collect: [ :aClass | aClass methodDict ]).
		
	#(	Display
		Smalltalk
		Transcript
		SourceFiles
		Processor
		SystemOrganization
		Sensor
		ActiveWorld) do: [:s | self analyzer considerGlobal: s].
		
	self assertSerializationEqualityOf: methodDicts.
! !

!FLFullBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/30/2011 23:36'!
testSmalltalkGlobalAssociation

	self analyzer considerGlobal: #Transcript.
	self assertSerializationIdentityOf: (Smalltalk globals associationAt: #Transcript)! !

FLSerializationTest subclass: #FLSequencedSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevelTests'!

!FLSequencedSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/31/2011 00:01'!
assertSequencedSerializationEqualityOf: aCollectionToSerialize
			
	| aSerializer fileName aMaterializer materializedCollection |
	fileName := 'sequencedSerializationTest.fuel'.
	aSerializer := FLSerializer newDefault.
	aMaterializer := FLMaterializer newDefault.

	FileStream forceNewFileNamed: fileName do: [:aWriteStream |
		aWriteStream binary.
		aCollectionToSerialize do: [:anObject |
			 aSerializer serialize: anObject on: aWriteStream ]].
	
	FileStream oldFileNamed: fileName do: [:aReadStream |
		aReadStream binary.
		materializedCollection := OrderedCollection new.
		[aReadStream atEnd] whileFalse: [
			materializedCollection add: 
				(aMaterializer materializeFrom: aReadStream) root ]].
	
	self assert: aCollectionToSerialize = materializedCollection.! !

!FLSequencedSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/31/2011 00:01'!
testMethodDictionaries

	self assertSequencedSerializationEqualityOf: (1 to: 50) asOrderedCollection.
! !

FLClassSerializationTest subclass: #FLCreateClassSerializationTest
	uses: FLTCreateClassOrTraitSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevelTests'!

!FLCreateClassSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of an entire class, in the cases in which the class *is not* present in the image at materialization time.!

FLCreateClassSerializationTest class
	uses: FLTCreateClassOrTraitSerializationTest classTrait
	instanceVariableNames: ''!

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 02:32'!
assertMaterializedHasCorrectFormat: aClass
	| materializedClass |	
	self analyzer considerInternalAll: (Array with: aClass).
	materializedClass := self resultOfSerializeAndMaterialize: aClass.
	
	self deny: aClass == materializedClass.
	self assert: aClass typeOfClass = materializedClass typeOfClass.
	self assert: aClass format = materializedClass format.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/7/2011 02:59'!
testCreateByteArrayWithInstance
	"Tests internal materialization of ByteArray together with an instance of it."
	
	| result |
	self analyzer considerInternalAll: (ByteArray withAllSuperclasses removeAll: Collection withAllSuperclasses; yourself).
	
	result := self resultOfSerializeAndMaterialize: (ByteArray with: 42).
	
	self assert: result class name = #ByteArray.
	self assert: result size = 1.
	self assert: result first = 42.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 19:46'!
testCreateClassAndMetaclass
	"Tests materialization a class or trait not defined in the image and that the metaclass is also created."

	| aClassOrTrait materializedClassOrTrait environment category name metaclassName metaclass |
	aClassOrTrait := self newClassOrTrait.
	environment := aClassOrTrait environment.
	category := aClassOrTrait category.
	name := aClassOrTrait name.
	metaclassName := aClassOrTrait class name.
	metaclass := aClassOrTrait class.
	
	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.
	
	self assert: environment == materializedClassOrTrait environment.
		"Classes are not direclty inserted in the system organization so we can only test #basicCategory and not #category."
	self assert: category = materializedClassOrTrait basicCategory.
	self assert: name = materializedClassOrTrait name.
	"Notice that the metaclass is serialized by Fuel and a new one will be created."
	self assert: metaclassName = materializedClassOrTrait class name.
	self deny: metaclass == materializedClassOrTrait class.

    "It is important to notice that (at this moment) Fuel does not add the materialized class or trait into Smalltalk globals"
    self should: [Smalltalk at: name] raise: Error.
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 03:12'!
testCreateClassWithClassSideVariable
	"Tests materialization a class not defined in the image, with a class-side instance variable."
	
	| aClass materializedClass |	
	aClass := self newClassOrTrait.
	aClass class addInstVarNamed: 'a'.
	aClass instVarNamed: 'a' put: #test.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.

	self assert: 1 = materializedClass class instVarNames size.
	self assert: (materializedClass class instVarNames includes: 'a').
	self assert: #test = (materializedClass instVarNamed: 'a').! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 02:42'!
testCreateClassWithClassVariable
	"Tests materialization a class not defined in the image, with a class variable."
	
	| aClass materializedClass |	
	aClass := self classFactory 
		newSubclassOf: Object 
		instanceVariableNames: '' 
		classVariableNames: 'ClassVariable'.
	aClass classPool at: 'ClassVariable' put: #test.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.

	self assert: 1 = materializedClass classVarNames size.
	self assert: (materializedClass classVarNames includes: #ClassVariable).
	self assert: #test = (materializedClass classPool at: #ClassVariable).! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 04:09'!
testCreateClassWithCreatedTrait
	"Tests materialization of a class and a trait not defined in the image. The class uses the trait."
	
	| aClass aTrait result materializedClass materializedTrait |	
	aClass := self newClassOrTrait.
	aClass compileSilently: 'fortyOne ^41'.
	aTrait := self newSecondaryTrait.
	aTrait compileSilently: 'fortyTwo ^42'.
	aClass addToComposition: aTrait.
	
	result := self resultOfSerializeRemoveAndMaterializeAll: (Array with: aClass with: aTrait).
	materializedClass := result first.
	materializedTrait := result second.

	self assert: result size = 2.
	self assert: (materializedClass traits first == materializedTrait).
	self assert: (materializedTrait users includes: materializedClass).
	self assert: materializedClass basicLocalSelectors size = 1.
	self assert: (materializedClass basicLocalSelectors includes: #fortyOne).
	self assert: materializedClass traitComposition transformations isEmpty not.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/7/2011 17:46'!
testCreateClassWithInstance
	"Tests materialization of a class who references an instance of itself."
	
	| aClass anInstance materializedClass materializedInstance |
	aClass := self newClassWithInstanceVariableNames: 'name'.
	aClass class instanceVariableNames: 'instance'.
	
	anInstance := aClass new
		instVarNamed: 'name' put: #testName;
		yourself.
	aClass 
		instVarNamed: 'instance' 
		put: anInstance.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.
	materializedInstance := materializedClass instVarNamed: 'instance'.
	
	self deny: aClass == materializedClass.
	self deny: anInstance == materializedInstance.
	self assert: #testName = (materializedInstance instVarNamed: 'name').! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 12/13/2011 20:44'!
testCreateClassWithSharedPool
	"Tests materialization of a class not defined in the image, with a shared pool (NOTE: it will be an identity problem if it isn't a Smalltalk global')"
	
	| aClass materializedClass |	
	aClass := self newClassOrTrait.
	aClass addSharedPool: TextConstants.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.

	self assert: 1 = (materializedClass sharedPools size).
	self assert: TextConstants == materializedClass sharedPools first.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 12/13/2011 21:25'!
testCreateClassWithSharedPoolAndMethodReferencinAClassVariableOfIt
	"Tests materialization of a class not defined in the image, with a shared pool and a compiled method that referers to a class side variable of the  sharedPool (NOTE: it will be an identity problem if it isn't a Smalltalk global')"
	
	| aClass materializedClass sharedPool anObject |	
	
	aClass := self newClassOrTrait.
	anObject := Object new.
	
	sharedPool := self newClassOrTraitWithSuperClass: SharedPool.
	sharedPool addClassVarNamed: 'ClassVar1'.
	sharedPool  class compile: 'classVar1: a  ClassVar1 := a. '.
	sharedPool  class compile: 'classVar1 ^  ClassVar1 '.
	sharedPool perform: #classVar1: with: anObject.
	
	aClass addSharedPool: sharedPool.	
	aClass compile: 'xxx ^ ClassVar1.'.
	aClass class compile: 'yyy ^ ClassVar1.'.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.

	self assert: 1 = (materializedClass sharedPools size).
	self assert: sharedPool == materializedClass sharedPools first.
	self assert:  ((materializedClass >> #xxx) literalAt: 1) value == (sharedPool classPool at: #ClassVar1).
	self assert:  ((materializedClass class >> #yyy) literalAt: 1) value == (sharedPool classPool at: #ClassVar1).
	self assert: (materializedClass new perform: #xxx) == anObject.
	self assert: (materializedClass perform: #yyy) == anObject.
	
	
 ! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 12/13/2011 20:44'!
testCreateClassWithSharedPoolAndMethodReferencingIt
	"Tests materialization of a class not defined in the image, with a shared pool and a compiled method that referers to an Association with such sharedPool (NOTE: it will be an identity problem if it isn't a Smalltalk global')"
	
	| aClass materializedClass sharedPool |	
	aClass := self newClassOrTrait.
	sharedPool := self newClassOrTraitWithSuperClass: SharedPool.
	aClass addSharedPool: sharedPool.
	aClass class compile: 'initialize ', sharedPool name asString, ' name.'.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.

	self assert: 1 = (materializedClass sharedPools size).
	self assert: sharedPool == materializedClass sharedPools first.
	self assert:  ((materializedClass class >> #initialize) literalAt: 2) value == sharedPool.
	
 ! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/7/2011 17:46'!
testCreateClassWithVariables
	"Tests materialization of a class not defined in the image, with instance variables."
	
	| aClass materializedClass |	
	aClass := self newClassWithInstanceVariableNames: 'var1 var2'.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.

	self assert: aClass instVarNames = materializedClass instVarNames.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 02:32'!
testCreateCompiledMethodClass
	"Tests materialization of a CompiledMethod class that is not defined in the image"
	self assertMaterializedHasCorrectFormat: FLCompiledMethodClassMock
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 03:19'!
testCreateHierarchy
	"Tests materialization of a hierarchy of classes that is not defined in the image"
	
	| serializedClasses materializedClasses a b c d |	
	a := self classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: ''.
	b := self classFactory newSubclassOf: a instanceVariableNames: '' classVariableNames: ''.
	c := self classFactory newSubclassOf: a instanceVariableNames: '' classVariableNames: ''.
	d := self classFactory newSubclassOf: c instanceVariableNames: '' classVariableNames: ''.
	serializedClasses := Array with: a with: b with: c with: d.
	
	materializedClasses := self resultOfSerializeRemoveAndMaterializeAll: serializedClasses.

	self assert: serializedClasses size = materializedClasses size.

	serializedClasses withIndexDo: [ :aClass :index | 
		self deny: aClass == (materializedClasses at: index) ].	

	self assert: Object = materializedClasses first superclass.
	self assert: materializedClasses first = materializedClasses second superclass.
	self assert: materializedClasses first = materializedClasses third superclass.
	self assert: materializedClasses third = materializedClasses fourth superclass.

	self assert: (Set with: materializedClasses second with: materializedClasses third) = materializedClasses first subclasses asSet.
	self assert: materializedClasses second subclasses isEmpty.
	self assert: (Set with: materializedClasses fourth) = materializedClasses third subclasses asSet.
	self assert: materializedClasses fourth subclasses isEmpty.
	"Notice that the instVar subclasses is only filled with classes inside the serialization. But for exernal classes Fuel does nothing. In this case,  FLStubA should NOT be added to Object subclasses."
	self deny: (Object subclasses includes: materializedClasses first).
	self deny: (Object subclasses identityIncludes: materializedClasses first).! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 03:23'!
testCreateHierarchyWithExistingClasses
	"Tests materialization of a hierarchy of classes that are not defined in the image and classes that indeed are defined"
	
	| serializedClasses materializedClasses a b c d |	
	a := self classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: ''.
	b := self classFactory newSubclassOf: a instanceVariableNames: '' classVariableNames: ''.
	c := self classFactory newSubclassOf: a instanceVariableNames: '' classVariableNames: ''.
	d := self classFactory newSubclassOf: Date instanceVariableNames: '' classVariableNames: ''.
	serializedClasses := Array with: a with: b with: c with: d.
	
	materializedClasses := self resultOfSerializeRemoveAndMaterializeAll: serializedClasses.

	self assert: serializedClasses size = materializedClasses size.

	serializedClasses withIndexDo: [ :aClass :index | 
		self deny: aClass == (materializedClasses at: index) ].	

	self assert: Object == materializedClasses first superclass.
	self assert: materializedClasses first == materializedClasses second superclass.
	self assert: materializedClasses first == materializedClasses third superclass.
	"Notice that the superclass of d is Date and since it is present in the image, we need exactly that instance."
	self assert: Date == materializedClasses fourth superclass.

	self assert: (Set with: materializedClasses second with: materializedClasses third) = materializedClasses first subclasses asSet.
	self assert: materializedClasses second subclasses isEmpty.
	self assert: materializedClasses third subclasses isEmpty.
	self assert: materializedClasses fourth subclasses isEmpty.
	
	self deny: (Object subclasses identityIncludes: materializedClasses first).
	self deny: (Date subclasses identityIncludes: materializedClasses fourth).! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 02:32'!
testCreateNormalClass
	"Tests materialization of a class that is not defined in the image"
	self assertMaterializedHasCorrectFormat: FLPair! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 12/13/2011 20:45'!
testCreateSharedPoolClass
	"Tests materialization of a shared pool class not defined in the image (NOTE: it will be an identity problem if it isn't a Smalltalk global')"
	
	|  materializedClass aClass sharedPool |
	sharedPool := self newClassOrTraitWithSuperClass: SharedPool.
	sharedPool addClassVarNamed: 'ClassVar1'.
	sharedPool  class compile: 'classVar1: a  ClassVar1 := a. '.
	sharedPool  class compile: 'classVar1 ^  ClassVar1 '.
	
	sharedPool perform: #classVar1: with: #foo.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: sharedPool.

	self assert: (materializedClass perform: #classVar1) = #foo.
	! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 02:32'!
testCreateVariableByteClass
	"Tests materialization of a variable byte class that is not defined in the image"
	self assertMaterializedHasCorrectFormat: FLVariableByteClassMock
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 02:32'!
testCreateVariableClass
	"Tests materialization of a variable class that is not defined in the image"
	self assertMaterializedHasCorrectFormat: FLVariableClassMock
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 02:32'!
testCreateVariableWordClass
	"Tests materialization of a variable word class that is not defined in the image"
	self assertMaterializedHasCorrectFormat: FLVariableWordClassMock! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 02:32'!
testCreateWeakClass
	"Tests materialization of a weak class that is not defined in the image"
	self assertMaterializedHasCorrectFormat: FLWeakClassMock! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 19:48'!
testCreateWithClassSideInitializeMethod
	"Tests materialization of a class-side #initialize method in a class or trait not defined in the image. It makes sure that Fuel doesn't call it automatically"

	| aClassOrTrait materializedClassOrTrait |
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait classSide compileSilently: 'initialize self error: ''Fuel should not automatically call #initialize'' ' classified: #test.

	self shouldnt: [materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait] raise: Error.
	self should: [ (self newInstanceFrom: materializedClassOrTrait) class perform: #initialize] raise: Error.

	self assert: (materializedClassOrTrait classSide includesSelector: #initialize).
	self assert: #(initialize) asSet = materializedClassOrTrait classSide localSelectors.
	self assert: #test = (materializedClassOrTrait classSide whichCategoryIncludesSelector: #initialize).
	
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 02:41'!
testCreateWithClassVariableAccessors
	"Tests materialization of a compiled method in a class not defined in the image. The class defines accessors for a class variable."
	
	| aClass materializedClass instance |	
	aClass := self classFactory 
		newSubclassOf: Object 
		instanceVariableNames: '' 
		classVariableNames: 'ClassVariable'.
	aClass 
		compileSilently: 'classVariable ^ClassVariable';
		compileSilently: 'classVariable: value ClassVariable := value'.
	(self newInstanceFrom: aClass) classVariable: 42.

	materializedClass := self resultOfSerializeRemoveAndMaterialize: aClass.
	instance := self newInstanceFrom: materializedClass.

	self assert: (materializedClass includesSelector: #classVariable).
	self assert: 42 = instance classVariable.
	instance classVariable: 0.
	self assert: 0 = instance classVariable.
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 19:32'!
testCreateWithNilEnvironment 
	"Tests materialization a class or trait not defined in the image, and with a nil in the instance variable 'environment ' "

	| aClassOrTrait materializedClassOrTrait |
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait environment: nil.
	
	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert:  materializedClassOrTrait basicEnvironment isNil.
! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 19:49'!
testCreateWithSmalltalkGlobalsEnvironment 
	"Tests materialization a class or trait not defined in the image, and with Smalltalk globals  in the instance variable 'environment '. In that case, the whole Smalltalk globals should NOT be fully serialized but instead just store a reference to it. "

	| aClassOrTrait materializedClassOrTrait category name |
	aClassOrTrait := self newClassOrTrait.
	aClassOrTrait environment: Smalltalk globals.
	category := aClassOrTrait category.
	name := aClassOrTrait name.
	
	materializedClassOrTrait := self resultOfSerializeRemoveAndMaterialize: aClassOrTrait.

	self assert:  materializedClassOrTrait basicEnvironment == Smalltalk globals.

	"Classes are not direclty inserted in the system organization so we can only test #basicCategory and not #category."
	self assert: category = materializedClassOrTrait basicCategory.
	self assert: name = materializedClassOrTrait name.

! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 03:20'!
testCreateWithSubclass
	"Tests materialization of a class that has a external subclass. Currently, this is the expected behaviour."
	
	| serializedClass aSubclass materializedClass |	
	serializedClass := self classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: ''.
	aSubclass := self classFactory newSubclassOf: serializedClass instanceVariableNames: '' classVariableNames: ''.
	
	materializedClass := self resultOfSerializeRemoveAndMaterialize: serializedClass.

	self assert: materializedClass subclasses size = 1.
	self assert: materializedClass subclasses first == aSubclass.! !

!FLCreateClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 10/22/2011 14:15'!
testMaterializationDoesNotModifySerializedClass
	"Tests that materialization does not change the source class."
	
	| aClass materializedClass |	
	aClass := self newClassWithInstanceVariableNames: 'var'.
	
	self analyzer considerInternalAll: (Array with: aClass).
	self serialize: aClass.
	aClass removeInstVarNamed: 'var'.
	materializedClass := self materialized.

	self assert: materializedClass instVarNames = #(var) description: 'The materialized class has the variable as the serialized class'.
	self assert: aClass instVarNames isEmpty description: 'Materialization should not modify the serialized class'.! !

FLTraitSerializationTest subclass: #FLCreateTraitSerializationTest
	uses: FLTCreateClassOrTraitSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevelTests'!

!FLCreateTraitSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of an entire trait, in the cases in which it *isnot* present in the image at materialization time.!

FLCreateTraitSerializationTest class
	uses: FLTCreateClassOrTraitSerializationTest classTrait
	instanceVariableNames: ''!
